package cn.com.qmth.examcloud.exchange.outer.service.impl.scorePush;

import java.io.IOException;
import java.net.URI;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.oe.marking.ExamRecordDto;
import cn.com.qmth.examcloud.commons.base.util.JsonUtil;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamScoreDataCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamScoreDataBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.FindExamScoreDataReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.FindExamScoreDataResp;
import cn.com.qmth.examcloud.exchange.outer.service.ScorePushService;

/**
 * @author  	chenken
 * @date    	2018年5月17日 上午11:01:45
 * @company 	QMTH
 * @description 石油大学推送成绩接口
 */
@Service("sydxScorePushService")
public class SydxScorePushServiceImpl implements ScorePushService{

    private static final Logger LOG = LoggerFactory.getLogger(SydxScorePushServiceImpl.class);

    @Value("${sydx.pushscore.url.schema}")
    private String pushUrlSchema;

    @Value("${sydx.pushscore.url.host}")
    private String pushUrlHost;

    @Value("${sydx.pushscore.url.port}")
    private String pushUrlPort;
    
	/*@Autowired
	private OutletScoreDao outletScoreDao;*/
	
	@Autowired
	private ExamScoreDataCloudService examScoreDataCloudService;
	
	@Override
	public void scorePush(ExamRecordDto examRecordDto) throws Exception {
		//有警告未审核
        if (examRecordDto.isWarn() && !examRecordDto.isAudit()) {
            return;
        }
        //违纪
  		if(examRecordDto.getInvigilatorOperation() == 1){
  			return;
  		}
        //考试记录是作废状态
  		if(StringUtils.isNotBlank(examRecordDto.getExamRecordStatus()) &&
  				"EXAM_INVALID".equals(examRecordDto.getExamRecordStatus())){
  			return;
  		}
		
  		Long scoreId = examRecordDto.getScoreId();
  		
		LOG.info("准备推送成绩：{}", scoreId);
		 
		FindExamScoreDataReq findExamScoreDataReq = new FindExamScoreDataReq();
		findExamScoreDataReq.setScoreId(scoreId);
		FindExamScoreDataResp findExamScoreDataResp =  examScoreDataCloudService.findExamScoreDataByScoreId(findExamScoreDataReq);
		List<ExamScoreDataBean> examScoreDataList = findExamScoreDataResp.getExamScoreDatas();
        //List<OutletScore> outletScores = outletScoreDao.sydxGetExamScoreById(scoreId);

        if (examScoreDataList == null || examScoreDataList.size() == 0) {
            LOG.info("{}成绩数据不存在", scoreId);
        }
        
        for(ExamScoreDataBean outletScore:examScoreDataList){
        	String courseCode = outletScore.getCourseCode();
        	//将Z000001变成000001
        	if(courseCode.startsWith("Z")||courseCode.startsWith("Q")||courseCode.startsWith("S")){
        		outletScore.setCourseCode(courseCode.substring(1, courseCode.length()));
        	}else{
        		outletScore.setCourseCode(courseCode);
        	}
        }

        CloseableHttpResponse httpResponse = null;
        CloseableHttpClient httpClient = null;
        try {
            String scoreStr = JsonUtil.toJson(examScoreDataList);
            LOG.info("成绩数据：{}", scoreStr);
            RequestConfig config = RequestConfig.custom()
                    .setConnectTimeout(5000)
                    .setConnectionRequestTimeout(5000)
                    .setSocketTimeout(5000)
                    .build();

            httpClient = HttpClientBuilder
                    .create()
                    .setDefaultRequestConfig(config)
                    .build();

            URI uri = new URIBuilder()
                    .setScheme(pushUrlSchema)
                    .setHost(pushUrlHost)
                    .setPort(Integer.parseInt(pushUrlPort))
                    .setPath("/entity/learning/interface/stuHomeworkScore_syncFinalExamScore.action")
                    .setParameter("loginId", "bjsy")
                    .setParameter("password", "1111")
                    .setParameter("examRemark", examScoreDataList.get(0).getExamName())
                    //.setParameter("examStuRemark", examScoreDataList.get(0).getStudentRemark())
                    .setParameter("scoreJsonStr", scoreStr)//通过查询参数的方式传递
                    .build();

            HttpPost httpPost = new HttpPost(uri);
//	            StringEntity data = new StringEntity(scoreStr);
//	            httpPost.setEntity(data);

            httpResponse = httpClient.execute(httpPost);
            HttpEntity entity = httpResponse.getEntity();
            String entityStr = EntityUtils.toString(entity);
            LOG.info("推送结果：{}", entityStr);
        } catch (Exception e) {
            LOG.error(scoreId + "成绩推送失败", e);
            e.printStackTrace();
        } finally {

            if (httpResponse != null) {
                try {
                    httpResponse.close();
                } catch (IOException e) {
                    e.printStackTrace();
                } finally {
                    httpResponse = null;
                }
            }

            if (httpClient != null) {
                try {
                    httpClient.close();
                } catch (IOException e) {
                    e.printStackTrace();
                } finally {
                    httpClient = null;
                }
            }
        }
	}

}

